@@ -25,7 +25,7 @@ from dagster_k8s.job import (
     get_user_defined_k8s_config,
 )
 
-from .config import CELERY_K8S_CONFIG_KEY, celery_k8s_executor_config
+from dagster_celery_k8s.config import CELERY_K8S_CONFIG_KEY, celery_k8s_executor_config
 
 
 class CeleryK8sRunLauncher(RunLauncher, ConfigurableClass):
@@ -158,7 +158,7 @@ class CeleryK8sRunLauncher(RunLauncher, ConfigurableClass):
         pod_name = job_name
         exc_config = _get_validated_celery_k8s_executor_config(run.run_config)
 
-        job_image_from_executor_config = exc_config.get("job_image")
+        job_image_from_executor_config = exc_config.get("job_image")  # pyright: ignore[reportOptionalMemberAccess]
 
         job_origin = cast(JobPythonOrigin, context.job_code_origin)
         repository_origin = job_origin.repository_origin
@@ -190,12 +190,6 @@ class CeleryK8sRunLauncher(RunLauncher, ConfigurableClass):
             job_image = job_image_from_executor_config
 
         job_config = self.get_k8s_job_config(job_image, exc_config)
-
-        self._instance.add_run_tags(
-            run.run_id,
-            {DOCKER_IMAGE_TAG: job_config.job_image},
-        )
-
         user_defined_k8s_config = get_user_defined_k8s_config(run.tags)
 
         from dagster._cli.api import ExecuteRunArgs
@@ -211,9 +205,9 @@ class CeleryK8sRunLauncher(RunLauncher, ConfigurableClass):
             "dagster/job": job_origin.job_name,
             "dagster/run-id": run.run_id,
         }
-        if run.external_job_origin:
+        if run.remote_job_origin:
             labels["dagster/code-location"] = (
-                run.external_job_origin.external_repository_origin.code_location_origin.location_name
+                run.remote_job_origin.repository_origin.code_location_origin.location_name
             )
 
         job = construct_dagster_k8s_job(
@@ -227,7 +221,13 @@ class CeleryK8sRunLauncher(RunLauncher, ConfigurableClass):
             env_vars=[{"name": "DAGSTER_RUN_JOB_NAME", "value": job_origin.job_name}],
         )
 
-        job_namespace = exc_config.get("job_namespace", self.job_namespace)
+        # Set docker/image tag here, as it can also be provided by `user_defined_k8s_config`.
+        self._instance.add_run_tags(
+            run.run_id,
+            {DOCKER_IMAGE_TAG: job.spec.template.spec.containers[0].image},
+        )
+
+        job_namespace = exc_config.get("job_namespace", self.job_namespace)  # pyright: ignore[reportOptionalMemberAccess]
 
         self._instance.report_engine_event(
             "Creating Kubernetes run worker job",
@@ -276,7 +276,7 @@ class CeleryK8sRunLauncher(RunLauncher, ConfigurableClass):
         check.str_param(run_id, "run_id")
 
         run = self._instance.get_run_by_id(run_id)
-        if not run:
+        if not run or run.is_finished:
             return False
 
         self._instance.report_run_canceling(run)
@@ -298,8 +298,7 @@ class CeleryK8sRunLauncher(RunLauncher, ConfigurableClass):
             else:
                 self._instance.report_engine_event(
                     message=(
-                        "Dagster Job was not terminated successfully; delete_job returned {}"
-                        .format(termination_result)
+                        f"Dagster Job was not terminated successfully; delete_job returned {termination_result}"
                     ),
                     dagster_run=run,
                     cls=self.__class__,
@@ -321,16 +320,16 @@ class CeleryK8sRunLauncher(RunLauncher, ConfigurableClass):
         check.str_param(run_id, "run_id")
 
         dagster_run = self._instance.get_run_by_id(run_id)
-        run_config = dagster_run.run_config
+        run_config = dagster_run.run_config  # pyright: ignore[reportOptionalMemberAccess]
         executor_config = _get_validated_celery_k8s_executor_config(run_config)
-        return executor_config.get("job_namespace", self.job_namespace)
+        return executor_config.get("job_namespace", self.job_namespace)  # pyright: ignore[reportOptionalMemberAccess]
 
     @property
     def supports_check_run_worker_health(self):
         return True
 
     def check_run_worker_health(self, run: DagsterRun):
-        job_namespace = _get_validated_celery_k8s_executor_config(run.run_config).get(
+        job_namespace = _get_validated_celery_k8s_executor_config(run.run_config).get(  # pyright: ignore[reportOptionalMemberAccess]
             "job_namespace", self.job_namespace
         )
         job_name = get_job_name_from_run_id(run.run_id)
@@ -340,6 +339,11 @@ class CeleryK8sRunLauncher(RunLauncher, ConfigurableClass):
             return CheckRunHealthResult(
                 WorkerStatus.UNKNOWN, str(serializable_error_info_from_exc_info(sys.exc_info()))
             )
+
+        if not status:
+            return CheckRunHealthResult(
+                WorkerStatus.UNKNOWN, f"K8s job {job_name} could not be found"
+            )
         if status.failed:
             return CheckRunHealthResult(WorkerStatus.FAILED, "K8s job failed")
         return CheckRunHealthResult(WorkerStatus.RUNNING)
@@ -365,11 +369,8 @@ def _get_validated_celery_k8s_executor_config(run_config):
         res.success,
         "Incorrect execution schema provided. Note: You may also be seeing this error "
         "because you are using the configured API. "
-        "Using configured with the {config_key} executor is not supported at this time, "
-        "and all executor config must be directly in the run config without using configured."
-        .format(
-            config_key=CELERY_K8S_CONFIG_KEY,
-        ),
+        f"Using configured with the {CELERY_K8S_CONFIG_KEY} executor is not supported at this time, "
+        "and all executor config must be directly in the run config without using configured.",
     )
 
     return res.value
